'use strict';

const 	path 		= require( 'path' )
// ,		validator	= require( 'validator' )
,		regexes		= {
                        'email' :  /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/
                    ,	'hostname' : /^[a-zA-Z](([-0-9a-zA-Z]+)?[0-9a-zA-Z])?(\.[a-zA-Z](([-0-9a-zA-Z]+)?[0-9a-zA-Z])?)*$/
                    }
;

let 	types 		= {
						'any'		: {
							'default'		: ''
						,	'validator'		: ( val ) => true
						}
					,	'string'		: {
							'default'		: ''
						,	'validator'		: ( val ) => typeof( val ) === 'string'
						}
					,	'number'		: {
							'default'		: 1
						,	'validator'		: ( val ) =>
												typeof( val ) === 'number'
												&&
												Number.isFinite( val )
												&&
												isNaN( val ) === false

						}
					,	'array'		: {
							'default'		: []
						,	'validator'		: ( val ) => Object.prototype.toString.call( val ) === '[object Array]'
						}
					,	'object'		: {
							'default'		: {}
						,	'validator'		: ( val ) => Object.prototype.toString.call( val ) === '[object Object]'
						}
					,	'date'		: {
							'default'		: new Date()
						,	'validator'		: ( val ) => {
												if ( Object.prototype.toString.call( val ) === '[object Date]' )
												{
													return true;
												}
												else
												{
													return validator.isDate( val )
												}
											}
						}
					,	'email'		: {
							'default'		: 'john@doe.com'
						,	'validator'		: ( val ) => regexes.email.test( val )
						}
					// ,	'url' : {}
					,	'hostname'		: {
							'default'		: 'google.com'
						,	'validator'		: ( val ) =>
											{	// https://github.com/zaggino/z-schema/blob/master/src/FormatValidators.js
												var valid = regexes.hostname.test(val);
												if (valid) {
													// the sum of all label octets and label lengths is limited to 255.
													if (val.length > 255) { return false; }
													// Each node has a label, which is zero to 63 octets in length
													var labels = val.split(".");
													for (var i = 0; i < labels.length; i++) { if (labels[i].length > 63) { return false; } }
												}
												return valid;
											}
					// ,	'float'
					// ,	'integer'
					}
					,	'boolean'		: {
							'default'		: true
						,	'validator'		: ( val ) => typeof( val ) === 'boolean'
					}
					,	'emptyString' 	: {
							'default'		: ''
						,	'validator'		: ( val ) => val === ''
					}
					,	'null'		 	: {
							'default'		: null
						,	'validator'		: ( val ) => val === null
					}
				}
;


types.bool	= types.boolean;
types.str 	= types.string;
types.num 	= types.number;
types.arr 	= types.array;
types.obj 	= types.object;

module.exports = types